/*
 * Created on Jun 11, 2004
 *
 * This file is licensed under the terms of the Artistic License as
 * described at http://www.opensource.org/licenses/artistic-license.php
 *
 * Use and/or redistribution under the terms of the GNU Lesser GPL
 * (http://www.opensource.org/licenses/lgpl-license.php) is also allowed.
 */
package com.blackperl.Perl;

/**
 * Interface expected by methods in the <code>Map</code> class.
 * 
 * When using the <code>map</code> function (see {@link Map}), the data
 * returned by the evaluation function may actually be more items than are
 * processed as inputs. This is a common idiom in Perl, often used in the
 * creation of associate arrays (the Perl equivalent of
 * <code>java.util.HashTable</code>). Because of this, <code>map</code> and
 * <code>grep</code> (see {@link Grep}) use different interfaces for the
 * specification of the evaluation objects they operate upon.
 * <p>
 * To allow a <code>map</code> operation to potentially generate more items
 * than it is fed, the signature of the <code>eval</code> method dictates that
 * the return value is an array. Though this may be inconvenient for code that
 * implements 1-to-1 mapping, it allows maximum flexibility. In cases where the
 * programmer knows for certain that the <code>map</code> usage will only need
 * to do a 1-to-1 data mapping, <code>map</code> can also accept a
 * {@link GrepBlock} instance.
 * <p>
 * The <code>MapBlock</code> interface faces the same block-based limitations
 * as <code>GrepBlock</code>. In particular, while this can be used in the
 * declaration of anonymous inner-classes on the fly, these classes are limited
 * in what values from the current lexical scope can be accessed. Only those
 * values that are declared both <b>final</b> and <b>static</b> may be
 * referenced in one of these inner-classes. The solution to this is to use the
 * interface in the declaration of a named inner-class, one that defines its
 * own members and methods. This can then be used to encapsulate the local
 * information that needs to be passed to the particular call to
 * <code>map</code>. For an example of this, see {@link GrepBlock}.
 * 
 * @author rjray
 * @version $Id: MapBlock.java,v 1.4 2004/07/20 08:31:17 rjray Exp $
 */
public interface MapBlock
{
    /**
     * When the <code>map</code> function is called with an object implementing
     * this interface, this is the function it will expect to be able to
     * invoke for each element in the list it also receives. The implementing
     * class should provide this such that it takes a single argument, and
     * returns an array of zero or more objects (or null, which is also
     * acceptable).
     * 
     * @param object    An arbitrary object passed in by <code>map</code>
     * @return  An array of arbitrary objects, based on the functionality this
     * method implements
     */
    Object[] eval(Object object);
}
