/*
 * Created on Jun 24, 2004
 *
 * This file is licensed under the terms of the Artistic License as
 * described at http://www.opensource.org/licenses/artistic-license.php
 *
 * Use and/or redistribution under the terms of the GNU Lesser GPL
 * (http://www.opensource.org/licenses/lgpl-license.php) is also allowed.
 */
package com.blackperl.Perl;

import java.util.Locale;

/**
 * Static-method implementation of the Perl <code>ucfirst</code> keyword.
 * 
 * The class provides several static methods to emulate Perl's
 * <code>ucfirst</code>. In keeping with the philosophy of not directly
 * replicating Java functionality, there is no implementation of
 * <code>lc</code>. Rather, the developer should use the
 * <code>toUpperCase</code> method of the <code>String</code> class.
 * The methods actually use the <code>toUpperCase</code> method
 * to do the needed conversions. Thus, each version
 * of <code>ucfirst</code> also has a form that takes a <code>Locale</code>
 * object as an additional parameter, to dictate the rules for converting the
 * characters to upper-case.
 * <p>
 * Some simple examples:
 * <pre>
 *     import com.blackperl.Perl.Ucfirst;
 * 
 *     String name = "rObErT";
 *     String bettername = Ucfirst.ucfirst(name.toLowerCase());
 *     // bettername == "Robert"
 * 
 *     name = "Fred";
 *     name = Ucfirst.ucfirst(name); // name is unchanged in this case
 * </pre>
 * This implementation departs slightly from the Perl model in the following
 * way: if the argument passed to the function is a <code>StringBuffer</code>
 * rather than a <code>String</code>, the argument itself is modified in-place.
 * The return value is still a stringified representation of the final value,
 * but unlike the previous example, the value passed in has been altered:
 * <pre>
 *     import com.blackperl.Perl.Lcfirst;
 * 
 *     StringBuffer name = new StringBuffer("barney");
 *     String bettername = Ucfirst.ucfirst(name);
 *     // bettername == name.toString() == "Barney"
 * </pre> 
 * All components in this package provide an <code>instance</code> method to
 * retrieve a singleton object which may be used to call the static methods,
 * if the programmer prefers using an object to static invocation.
 * <p>
 * If this is the JDK 1.5 ("Tiger") edition of the package, this class is
 * suitable for use via static import:
 * <pre>
 *     import com.blackperl.Perl.Ucfirst.ucfirst;
 * 
 *     String name = "rObErT";
 *     String bettername = ucfirst(name.toLowerCase());
 *     // bettername is now "Robert".
 * </pre>
 * 
 * @author rjray
 * @version $Id: Ucfirst.java,v 1.6 2004/07/31 12:27:40 rjray Exp $
 */
public final class Ucfirst
{
    private static Ucfirst singleton;
    
    /**
     * Basic constructor, explicitly declared here as <code>private</code> to
     * prevent other classes instantiating this class.
     */
    private Ucfirst()
    {
        super();
    }
    
    /**
     * The <code>instance</code> method is used to retrieve the
     * <code>Ucfirst</code> singleton that applications can use in lieu of
     * invoking the methods statically.
     * 
     * @return  Singleton instance of the Ucfirst class
     */
    public synchronized static Ucfirst instance()
    {
        if (singleton == null)
        {
            singleton = new Ucfirst();
        }
        
        return(singleton);
    }

    /**
     * Take the <code>String</code> parameter and return a new
     * <code>String</code> whose first character has been made upper-case, if
     * the character is in fact alphabetic. This uses the default locale to
     * determine the correct way to convert to upper-case. Since the argument
     * is immutable, it itself is not changed.
     *  
     * @param string    The <code>String</code> to convert (not changed)
     * @return  The converted value
     */
    public static String ucfirst(String string)
    {
        return ucfirst(string, Locale.getDefault());
    }
    
    /**
     * Take the <code>String</code> parameter and return a new
     * <code>String</code> whose first character has been made upper-case, if
     * the character is in fact alphabetic. The <code>Locale</code> parameter
     * determines the correct way to convert to upper-case. Since the argument
     * is immutable, it itself is not changed.
     * 
     * @param string    The <code>String</code> to convert (not changed)
     * @param locale    A <code>Locale</code> object for upper-case rules
     * @return  The converted value
     */
    public static String ucfirst(String string, Locale locale)
    {
        // Simple: Do a toUpperCase and extract the first character, then use
        // that as a replacement in the return value. Should the first char not
        // actually be alphabetic, it won't have changed and no harm is done.
        String retval;
        
        retval = string.toUpperCase(locale).substring(0, 1) +
            string.substring(1);
        
        return retval;
    }
    
    /**
     * This form is a slight departure from the Perl equivalent, in that it
     * actually modifies the argument in-place. The first character is
     * converted to upper-case using the default locale rules (if the character
     * is not alphabetic, it is unchanged). Returns a <code>String</code> copy
     * of the converted value.
     * 
     * @param buffer    The <code>StringBuffer</code> to operate on
     * @return  A <code>String</code> copy of the converted value
     */
    public static String ucfirst(StringBuffer buffer)
    {
        return ucfirst(buffer, Locale.getDefault());
    }
    
    /**
     * This form is a slight departure from the Perl equivalent, in that it
     * actually modifies the argument in-place. The first character is
     * converted to upper-case using the locale rules defined in the
     * <code>Locale</code> parameter (if the character is not alphabetic, it
     * is unchanged). Returns a <code>String</code> copy of the converted
     * value.
     * 
     * @param buffer    The <code>StringBuffer</code> to operate on
     * @param locale    A <code>Locale</code> object for upper-case rules
     * @return  A <code>String</code> copy of the converted value
     */
    public static String ucfirst(StringBuffer buffer, Locale locale)
    {
        // Slightly easier than the String form, since we can manipulate a
        // specific character in the buffer itself.
        buffer.replace(0, 1, buffer.toString().toUpperCase(locale)
            .substring(0, 1));
            
        return buffer.toString();
    }
}
